import hashlib
import math
import time
from datetime import datetime as dtm, timedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def myFunc(e):
    return e["start"]


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        uprn = kwargs.get("uprn")
        check_uprn(uprn)
        today = int(datetime.now().timestamp()) * 1000
        dateforurl = datetime.now().strftime("%Y-%m-%d")
        dateforurl2 = (datetime.now() + timedelta(days=42)).strftime("%Y-%m-%d")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)",
        }
        requests.packages.urllib3.disable_warnings()

        # Get variables for workings
        response = requests.get(
            f"https://cms.northnorthants.gov.uk/bin-collection-search/calendarevents/{uprn}/{dateforurl}/{dateforurl2}",
            headers=headers,
        )
        if response.status_code != 200:
            raise ValueError("No bin data found for provided UPRN..")

        json_response = json.loads(response.text)

        output_dict = [
            x
            for x in json_response
            if int("".join(filter(str.isdigit, x["start"]))) >= today
        ]

        output_json = output_dict
        output_json.sort(key=myFunc)

        i = 0
        while i < len(output_json):
            sov = output_json[i]["title"].lower()
            if "recycling" in sov:
                bin_type = "Recycling"
            elif "garden" in sov:
                bin_type = "Garden"
            elif "refuse" in sov:
                bin_type = "General"
            else:
                bin_type = "Unknown"
            dateofbin = int("".join(filter(str.isdigit, output_json[i]["start"])))
            day = dtm.fromtimestamp(dateofbin / 1000)
            collection_data = {
                "type": bin_type,
                "collectionDate": day.strftime(date_format),
            }
            data["bins"].append(collection_data)
            i += 1

        return data



import pandas as pd
import time
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def get_data(self, df) -> dict:
        # Create dictionary of data to be returned
        data = {"bins": []}

        # Output collection data into dictionary
        for i, row in df.iterrows():
            dict_data = {
                "type": row["Collection Name"],
                "collectionDate": row["Next Collection Due"],
            }

            data["bins"].append(dict_data)

        return data

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://chiltern.gov.uk/collection-dates"

            # Assign user info
            user_postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # Enter postcode in text box and wait
            inputElement_pc = driver.find_element(
                By.ID,
                "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_ADDRESSSELECTIONPOSTCODE",
            )
            inputElement_pc.send_keys(user_postcode)
            inputElement_pc.send_keys(Keys.ENTER)

            time.sleep(4)

            # Select address from dropdown and wait
            inputElement_ad = Select(
                driver.find_element(
                    By.ID,
                    "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_ADDRESSSELECTIONADDRESS",
                )
            )

            inputElement_ad.select_by_visible_text(user_paon)

            time.sleep(4)

            # Submit address information and wait
            inputElement_bn = driver.find_element(
                By.ID, "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_NAV1_NEXT"
            ).click()

            time.sleep(4)

            # Read next collection information into Pandas
            table = driver.find_element(
                By.ID, "COPYOFECHOCOLLECTIONDATES_PAGE1_DATES2"
            ).get_attribute("outerHTML")
            df = pd.read_html(table, header=[1])
            df = df[0]

            # Parse data into dict
            data = self.get_data(df)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            bin_data_dict = {"bins": []}
            collections = []
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            data = {"bins": []}

            # Get our initial session running
            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 30)
            postcode = wait.until(
                EC.presence_of_element_located((By.XPATH, '//*[@id="pPostcode"]'))
            )

            postcode.send_keys(user_postcode)
            postcode_search_btn = wait.until(
                EC.element_to_be_clickable((By.CLASS_NAME, "searchbox_submit"))
            )
            postcode_search_btn.send_keys(Keys.ENTER)
            # Wait for the 'Select your property' dropdown to appear and select the first result
            dropdown = wait.until(EC.element_to_be_clickable((By.ID, "uprn")))

            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(user_uprn))
            checkbox = wait.until(EC.presence_of_element_located((By.ID, "gdprTerms")))
            checkbox.send_keys(Keys.SPACE)
            get_bin_data_btn = wait.until(
                EC.element_to_be_clickable((By.CLASS_NAME, "searchbox_submit"))
            )
            get_bin_data_btn.send_keys(Keys.ENTER)
            # Make a BS4 object
            results = wait.until(EC.presence_of_element_located((By.ID, "collection")))
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            data = {"bins": []}

            # Get collections
            row_index = 0
            for row in soup.find("table", {"id": "collection"}).find_all("tr"):
                # Skip headers row
                if row_index < 1:
                    row_index += 1
                    continue
                else:
                    # Get bin info
                    bin_info = row.find_all("td")
                    # Get the bin type
                    bin_type = bin_info[0].find("strong").get_text(strip=True)
                    # Get the collection date
                    collection_date = ""
                    for p in bin_info[2].find_all("p"):
                        if "your next collection" in p.get_text(strip=True):
                            collection_date = datetime.strptime(
                                " ".join(
                                    p.get_text(strip=True)
                                    .replace("will be your next collection.", "")
                                    .split()
                                ),
                                "%A %d %B %Y",
                            )

                    if collection_date != "":
                        # Append the bin type and date to the data dict
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": collection_date.strftime(date_format),
                        }
                        data["bins"].append(dict_data)

                    row_index += 1

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        json_result = json.loads(page.text)

        data = {"bins": []}

        for collection in json_result["collections"]:
            bin_type = collection["service"]
            bin_collection = collection["date"]  # Date format is 14/12/2023 00:00:00

            dict_data = {
                "type": bin_type.replace(" Collection Service", " Bin"),
                "collectionDate": datetime.strptime(
                    bin_collection, "%d/%m/%Y %H:%M:%S"
                ).strftime(date_format),
            }

            data["bins"].append(dict_data)

        return data



from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import pandas as pd
import urllib.request


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            page = f"https://www.eastriding.gov.uk/environment/bins-rubbish-recycling/bins-and-collections/bin-collection-dates/"

            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            wait = WebDriverWait(driver, 60)

            try:
                accept_cookies = wait.until(
                    EC.element_to_be_clickable(
                        (By.XPATH, '//*[@id="er-cookie-placeholder-settings"]/div/a[1]')
                    )
                )

                accept_cookies.click()
            except:
                print(
                    "Cookies acceptance element not found or clickable within the specified time."
                )
                pass

            expand_postcode_box = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//a[@href='#when-and-where-should-i-put-my-bin-out']")
                )
            )

            expand_postcode_box.click()

            postcode_box = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//input[@placeholder='Enter your postcode']")
                )
            )
            postcode_box.send_keys(user_postcode)

            postcode_search_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, "//button[text()='Search']"))
            )
            postcode_search_btn.send_keys(Keys.ENTER)

            dropdown = wait.until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        "//div[@class='er-select-wrapper']/select[@class='dropdown']",
                    )
                )
            )

            options_present = wait.until(
                EC.presence_of_all_elements_located(
                    (By.CSS_SELECTOR, "select.dropdown option")
                )
            )
            drop = Select(dropdown)
            drop.select_by_visible_text(str(user_paon))

            results_present = wait.until(
                EC.presence_of_element_located(
                    (
                        By.CLASS_NAME,
                        "results",
                    )
                )
            )

            data = {"bins": []}  # dictionary for data
            soup = BeautifulSoup(driver.page_source, "html.parser")

            bin_types = {}  # Dictionary to store bin types

            # Extract bin types from header elements
            header_elements = soup.find_all(
                "li", class_=lambda x: x and x.startswith("header")
            )
            for header_element in header_elements:
                bin_type = header_element.get_text(strip=True)
                bin_class = [
                    cls for cls in header_element.get("class") if cls != "header"
                ]
                if bin_class:
                    bin_types[bin_class[0]] = bin_type

            # Extract collection dates and associate them with respective bin types
            date_elements = soup.find_all(
                "li", class_=lambda x: x and x.startswith("date")
            )
            for date_element in date_elements:
                bin_class = [cls for cls in date_element.get("class") if cls != "date"]
                if bin_class:
                    bin_type = bin_types.get(bin_class[0])
                    span_text = date_element.find("span").get_text(strip=True)
                    collection_date = datetime.strptime(span_text, "%a, %d %b %Y")
                    data["bins"].append(
                        {
                            "type": bin_type,
                            "collectionDate": collection_date.strftime(date_format),
                        }
                    )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from time import sleep
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait, Select

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(
                "https://my.eastsuffolk.gov.uk/service/Bin_collection_dates_finder"
            )

            # Wait for iframe to load and switch to it
            WebDriverWait(driver, 30).until(
                EC.frame_to_be_available_and_switch_to_it((By.ID, "fillform-frame-1"))
            )

            # Wait for postcode entry box
            postcode = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "alt_postcode_search"))
            )
            # Enter postcode
            postcode.send_keys(user_postcode.replace(" ", ""))

            # Wait for address selection dropdown to appear
            address = Select(
                WebDriverWait(driver, 10).until(
                    EC.visibility_of_element_located((By.ID, "alt_choose_address"))
                )
            )

            # Wait for spinner to disappear (signifies options are loaded for select)
            WebDriverWait(driver, 10).until(
                EC.invisibility_of_element_located((By.CLASS_NAME, "spinner-outer"))
            )

            # Sometimes the options aren't fully there despite the spinner being gone, wait another 2 seconds.
            sleep(2)

            # Select address by UPRN
            address.select_by_value(user_uprn)

            # Wait for spinner to disappear (signifies data is loaded)
            WebDriverWait(driver, 10).until(
                EC.invisibility_of_element_located((By.CLASS_NAME, "spinner-outer"))
            )

            sleep(2)

            # Find data table
            data_table = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//div[@data-field-name="collection_details"]/div[contains(@class, "fieldContent")]/div[contains(@class, "repeatable-table-wrapper")]',
                    )
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(
                data_table.get_attribute("innerHTML"), features="html.parser"
            )

            data = {"bins": []}

            rows = soup.find("table").find("tbody").find_all("tr")
            for row in rows:
                cols = row.find_all("td")
                bin_type = cols[2].find_all("span")[1].text.title()
                collection_date = cols[3].find_all("span")[1].text
                collection_date = datetime.strptime(
                    collection_date, "%d/%m/%Y"
                ).strftime(date_format)
                dict_data = {"type": bin_type, "collectionDate": collection_date}
                data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_uprn = kwargs.get("uprn")
            check_uprn(user_uprn)

            user_postcode = kwargs.get("postcode")
            check_postcode(user_postcode)
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            data = {"bins": []}

            # Get our initial session running
            page = "https://carehomes.bolton.gov.uk/bins.aspx"

            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property
            wait = WebDriverWait(driver, 30)

            pc_search_box = wait.until(
                EC.presence_of_element_located((By.ID, "txtPostcode"))
            )

            pc_search_box.send_keys(user_postcode)

            pcsearch_btn = wait.until(EC.element_to_be_clickable((By.ID, "btnSubmit")))

            pcsearch_btn.click()

            # Wait for the 'Select your property' dropdown to appear and select the first result
            dropdown = wait.until(EC.element_to_be_clickable((By.ID, "ddlAddresses")))

            dropdown_options = wait.until(
                EC.presence_of_element_located((By.XPATH, "//select/option[1]"))
            )
            time.sleep(1)
            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(user_uprn))
            dropdown_options = wait.until(
                EC.presence_of_element_located((By.ID, "pnlStep3"))
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            collections = []

            # Find section with bins in
            sections = soup.find_all("div", {"class": "bin-info"})

            # For each bin section, get the text and the list elements
            for item in sections:
                words = item.find_next("strong").text.split()[2:4]
                bin_type = " ".join(words).capitalize()
                date_list = item.find_all("p")
                for d in date_list:
                    next_collection = datetime.strptime(d.text.strip(), "%A %d %B %Y")
                    collections.append((bin_type, next_collection))

            # Sort the text and list elements by date
            ordered_data = sorted(collections, key=lambda x: x[1])

            # Put the elements into the dictionary
            for item in ordered_data:
                dict_data = {
                    "type": item[0],
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


def get_month_number(month_name):
    return datetime.strptime(month_name, "%B").month


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # UPRN is street id here
            uprn = kwargs.get("uprn")
            check_uprn(uprn)
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 30)
            roadid = wait.until(
                EC.presence_of_element_located((By.XPATH, '//select[@name="roadID"]'))
            )
            dropdown = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//select[@name="roadID"]'))
            )

            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(uprn))
            search_btn = wait.until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//*[@id="wasteCalendarContainer"]/form/input[@value="Search"]',
                    )
                )
            )
            search_btn.click()

            results = wait.until(
                EC.presence_of_element_located((By.ID, "wasteCalendarContainer"))
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            data = {"bins": []}

            # Find all calendar containers
            calendar_containers = soup.select(".calendarContainer > .calendarContainer")

            # List to store dictionaries for each item
            data_list = []

            # Iterate through each calendar container
            for container in calendar_containers:
                # Find the header (h2 element) inside the current calendar container
                header = container.find("h2")

                # Extract month and year from the header
                if header:
                    month_year = header.get_text(strip=True)
                    month, year = month_year.split()

                    # Find all elements with class "pink" and "normal" inside the current calendar container
                    pink_days = container.find_all("td", class_="pink")
                    normal_days = container.find_all("td", class_="normal")
                    month_number = get_month_number(month)
                    # Extract the dates for "pink" and "normal" days and create dictionary items
                    for date in pink_days + normal_days:
                        day = date.get_text().zfill(2)
                        formatted_date = f"{day}/{month_number:02d}/{year}"

                        # Build dictionary for each item
                        dict_data = {
                            "type": "Pink" if date["class"][0] == "pink" else "Normal",
                            "collectionDate": formatted_date,
                        }
                        data["bins"].append(dict_data)
                else:
                    print("Invalid calendar format encountered.")
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests, re, urllib.parse

from datetime import datetime, timedelta
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        user_postcode = kwargs.get("postcode")
        user_number = kwargs.get("paon")

        user_address = "{} {}".format(user_number, user_postcode)
        user_address = urllib.parse.quote(user_address)

        # This first URL checks against a string represenging the users address and returns values used for a second lookup.
        stage1_url = "https://wlnk.statmap.co.uk/map/Cluster.svc/findLocation?callback=getAddressesCallback1702938375023&script=%5CCluster%5CCluster.AuroraScript%24&address={}".format(
            user_address
        )

        address_data = requests.get(stage1_url).text

        # Strip data and parse the JSON
        address_data = json.loads(
            re.sub("getAddressesCallback[\d]+\(", "", address_data)[:-2]
        )

        if address_data["TotalHits"] == 0:
            raise Exception(
                "No address found for string {}. See Wiki".format(user_address)
            )
        elif address_data["TotalHits"] != 1:
            # Multiple hits returned. Lets pick the first one. We could raise an exception here if this causes problems.
            pass

        # Pull out the address data needed for the next step
        address_id = address_data["Locations"][0]["Id"]
        address_x = address_data["Locations"][0]["X"]
        address_y = address_data["Locations"][0]["Y"]

        stage2_url = "https://wlnk.statmap.co.uk/map/Cluster.svc/getpage?script=\Cluster\Cluster.AuroraScript$&taskId=bins&format=js&updateOnly=true&query=x%3D{}%3By%3D{}%3Bid%3D{}".format(
            address_x, address_y, address_id
        )

        bin_query = requests.get(stage2_url).text

        # Test that what we got is good
        if "injectCss" not in bin_query:
            raise Exception(
                "Error. Data has not been returned correctly. Please raise an issue on the GitHub page"
            )

        # Return only the HTML contained within the Javascript function payload.
        pattern = 'document\.getElementById\("DR1"\)\.innerHTML="(.+)";'

        bin_html = re.findall(pattern, bin_query)

        if len(bin_html) != 1:
            # This exception is raised if the regular expression above finds anything other than one expected match.
            raise Exception(
                "Incorrect number of matches found during phase 2 search. Please raise an issue on the Github page"
            )

        # Some silly python foo required here to unescape the unicode contained.
        bin_html = bin_html[0].encode().decode("unicode-escape")

        soup = BeautifulSoup(bin_html, "html.parser")

        collection_rows = soup.find("li", {"class": "auroraListItem"}).find_all("li")

        for row in collection_rows:
            # Get bin type
            bin_type = row.find("span").text

            # Get bin date
            bin_date_text = row.text
            pattern = "\d+\/\d+"
            bin_dates = re.findall(pattern, bin_date_text)

            input_date_format = "%d/%m"

            for bin_date in bin_dates:
                # The date returned from the webpage only gives DD/MM. So we need to add a year, but we can't simply add this year otherwise we would get it wrong at the end of the year. So we will test to see if the returned date + this year is in the future. If not, add next years date.
                bin_dt = datetime.strptime(bin_date, input_date_format)
                bin_dt = bin_dt.replace(year=datetime.now().year)

                if (
                    bin_dt.date() == datetime.today().date()
                ):  # Check if date is today. This is OK
                    pass
                elif (
                    bin_dt.date() < datetime.today().date()
                ):  # Check if the date is in the past. If so, increment the year
                    bin_dt = bin_dt.replace(year=bin_dt.year + 1)
                elif (
                    bin_dt.date() > datetime.today().date()
                ):  # Check if date is in the future. This is OK
                    pass
                else:
                    raise Exception(
                        "Date issue has occured. This should never happen. Please raise a bug in GitHub"
                    )

                dict_data = {
                    "type": bin_type,
                    "collectionDate": bin_dt.strftime(date_format),
                }

                data["bins"].append(dict_data)

        return data
